function f_rec = gerchberg_saxton(f_init,fpsi,fphi,psi,phi,Kit)
% This function attempts to reconstruct a function from the modulus
% of its wavelet transform by alternate projections
    
% Inputs :
% - f_init : initial guess (optional)
% - fpsi : modulus of the wavelet transform
% - fphi : low frequency
% - [psi,phi] : wavelets
% - Kit : number of iterations (optional)
 
if (nargin<6)
    Kit = 200 ;
end
N = size(fpsi,2)/2 ;
if (numel(f_init)==0)
    f_init = zeros(1,N) ;
end
    
f_rec = f_init ;

[phi_conj,psi_conj] = dual_wavelets(phi,psi) ;

for k=1:Kit
    
    % Project the wavelet transform of f_rec on the set of
    % functions whose moduli correspond to the desired ones
    psi_rec = wavelet_transform(f_rec,psi,phi) ;
    psi_rec = fpsi.*psi_rec./(abs(psi_rec)+1e-13) ;
    
    % Project [psi_rec,phi_rec] on the set of admissible wavelet
    % transforms
    ff_rec = phi_conj.*fft(fphi) + ...
             sum(fft(psi_rec,[],2).*psi_conj,1) ;
    f_rec = ifft(ff_rec(1:N)) ;    
    
end